function getParamFromUrl (name, defaultValue) {
    let val = location.search.match(new RegExp('[?&]' + name + '=([^&]+)'));
    return val ? decodeURIComponent(val[1].replace(/\+/g, '%20')) : defaultValue;
}

function initLanguage(path) {
    const LANGUAGE = [
        'en',
        'zh-hans',
        'zh-hant',
    ];
    let lang = getParamFromUrl('lang', '');
    if (LANGUAGE.indexOf(lang) === -1) {
        lang = 'en';
    }
    const langNode = document.createElement('script');
    langNode.src = path + lang + '.js';
    document.head.appendChild(langNode);
}

initLanguage('../entitytree/msg/');
initLanguage('../console/msg/');
initLanguage('msg/');

window.addEventListener('load', function () {
    EntityTree.inject("entity-tree-area");
    Console.inject('console-area');

    const tags = document.getElementsByClassName('switch-button');
    WidgetFactory.listenTooltip(tags[0], Msg.TOOLTIP_ENTITY_TREE);
    WidgetFactory.listenTooltip(tags[1], Msg.TOOLTIP_CONSOLE);

    selectEntityTree();
    window.wxConsoleLoaded && window.wxConsoleLoaded();
});

function selectEntityTree () {
    const entityTree = document.getElementById('entity-tree-area');
    const console = document.getElementById('console-area');
    entityTree.style.display = 'unset';
    console.style.display = 'none';
    const tags = document.getElementsByClassName('switch-button');
    tags[0].addClassName('selected');
    tags[1].removeClassName('selected');
}

function selectConsole () {
    const entityTree = document.getElementById('entity-tree-area');
    const console = document.getElementById('console-area');
    entityTree.style.display = 'none';
    console.style.display = 'unset';

    const tags = document.getElementsByClassName('switch-button');
    tags[0].removeClassName('selected');
    tags[1].addClassName('selected');
}

window.wxSelectLanguage = function (newLang) {
    let search = window.location.search;
    if(search.length <= 1)
        search = '?lang=' + newLang;
    else if(search.match(/[?&]lang=[^&]*/))
        search = search.replace(/([?&]lang=)[^&]*/, '$1' + newLang);
    else
        search = search.replace(/\?/, '?lang=' + newLang + '&');

    window.location = window.location.protocol + '//' +
        window.location.host + window.location.pathname + search;
};