#ifdef GL_ES
//precision mediump float;
//precision mediump int;
precision highp float;
precision highp int;
#endif

////////////////////////////////////////////////////////////////////////////////
//uniform float msl;

uniform int SpaceMask;//water, earth air, magma, and etc. 0 is the normal air.
uniform vec4 clipPlane;

varying vec4 worldPosition;


uniform sampler2D shadow_map1;
uniform sampler2D shadow_map2;
varying vec4 shadow_coord1;
varying vec4 shadow_coord2;
varying float camera_distance;


//==============================================================================
uniform sampler2D sampler;
uniform sampler2D sky_sampler;
uniform float timer;
uniform float daylight;
uniform int ortho;

//------------------------------------------------------------------------------
varying vec2 fragment_uv;
varying float fragment_ao;
varying float fragment_light;
varying float fog_factor;
varying float fog_height;
varying float diffuse;

//const float pi = 3.14159265;

////////////////////////////////////////////////////////////////////////////////

float lookup(sampler2D shadow_map, vec4 coord, vec2 offset, float bias) {
	float z = texture2D(shadow_map, coord.xy + offset / 4096.0).r;
	return float(z > coord.z - bias);
}


vec2 random_offset(vec4 seed) {
	float angle = fract(sin(dot(seed, vec4(12.9898, 78.233, 45.164, 94.673)))
		* 43758.5453) * radians(360.0);
	return vec2(cos(angle), sin(angle)) * 2.0;
}

////////////////////////////////////////////////////////////////////////////////
void main() {
	if (dot(worldPosition, clipPlane) < 0.0) {
		discard;
	}
	//if (SpaceMask == 1) {//TODO: fog when in water. too bright
	//	fog_factor *= 100000.0;
	//}

	//--------------------------------------------------------------------------
	vec3 color = vec3(texture2D(sampler, fragment_uv));
    if (color == vec3(1.0, 0.0, 1.0)) {
        discard;
    }
    bool cloud = color == vec3(1.0, 1.0, 1.0);
    if (cloud && bool(ortho)) {
        discard;
    }
    float df = cloud ? 1.0 - diffuse * 0.2 : diffuse;
    float ao = cloud ? 1.0 - (1.0 - fragment_ao) * 0.2 : fragment_ao;
    ao = min(1.0, ao + fragment_light);
    df = min(1.0, df + fragment_light);
    float value = min(1.0, daylight + fragment_light);
    vec3 light_color = vec3(value * 0.3 + 0.2);
    vec3 ambient = vec3(value * 0.3 + 0.2);

	/*use shadowmap*/
	float visibility = 1.0;
	if (camera_distance < 16.0) {
		float sum = 0.0;
		const int count = 4;
		for (int i = 0; i < count; i++) {
			vec2 offset = random_offset(vec4(gl_FragCoord.xyz, i));
			sum += lookup(shadow_map1, shadow_coord1, offset, 0.0001);
		}
		visibility = 0.5 + sum / (float(count) * 2.0);
	}
	else if (camera_distance < 128.0) {
		float sum = lookup(shadow_map2, shadow_coord2, vec2(0.0), 0.0008);
		visibility = 0.5 + sum / 2.0;
	}

	vec3 light = (ambient + light_color * df) * visibility;
    color = clamp(color * light * ao, vec3(0.0), vec3(1.0));
    vec3 sky_color = vec3(texture2D(sky_sampler, vec2(timer, fog_height)));
    color = mix(color, sky_color, fog_factor);
	gl_FragColor = vec4(color, 1.0);

	//--------------------------------------------------------------------------
	if (SpaceMask == 1) {//water
		gl_FragColor[2] = clamp(gl_FragColor[2]+0.3, 0.0, 1.0);
	}
}
