#ifdef GL_ES
//precision mediump float;
//precision mediump int;
precision highp float;
precision highp int;
#endif

////////////////////////////////////////////////////////////////////////////////
uniform mat4 mmodel;

varying vec4 worldPosition;

uniform mat4 shadow_map_matrix1;
uniform mat4 shadow_map_matrix2;
varying vec4 shadow_coord1;
varying vec4 shadow_coord2;
varying float camera_distance;


//==============================================================================
uniform mat4 matrix;
uniform vec3 camera;
uniform float fog_distance;
uniform int ortho;

attribute vec4 position;
attribute vec3 normal;
attribute vec4 uv;

//------------------------------------------------------------------------------
varying vec2 fragment_uv;
varying float fragment_ao;
varying float fragment_light;
varying float fog_factor;
varying float fog_height;
varying float diffuse;

const float pi = 3.14159265;
vec3 light_direction = normalize(vec3(-1.0, 1.0, -1.0));

////////////////////////////////////////////////////////////////////////////////
void main() {
#if 1
	worldPosition = mmodel * position;
#else//1
	gl_ClipDistance[0] = dot(mmodel*position, clipPlane);
	gl_ClipDistance[0] = -1;
#endif//1

	//--------------------------------------------------------------------------
	gl_Position = matrix * position;

    fragment_uv = uv.xy;
    fragment_ao = 0.3 + (1.0 - uv.z) * 0.7;
    fragment_light = uv.w;
    diffuse = max(0.0, dot(normal, light_direction));
    if (bool(ortho)) {
        fog_factor = 0.0;
        fog_height = 0.0;
    }
    else {
		shadow_coord1 = (shadow_map_matrix1 * worldPosition + 1.0) / 2.0;
		shadow_coord2 = (shadow_map_matrix2 * worldPosition + 1.0) / 2.0;
        camera_distance = distance(camera, vec3(worldPosition));

		fog_factor = pow(clamp(camera_distance / fog_distance, 0.0, 1.0), 4.0);
        float dy = worldPosition.y - camera.y;
        float dx = distance(worldPosition.xz, camera.xz);
        fog_height = (atan(dy, dx) + pi / 2.0) / pi;
    }
}
