import woxiao
import forge
import util
import wx
import vmath
import math
import time

ShowDialog = forge.ShowDialog
ShowQuestionDialog = forge.ShowQuestionDialog
ShowInputDialog = forge.ShowInputDialog
ShowMessageBox = forge.ShowMessageBox
ShowContactBook = forge.ShowContactBook
ContactBookAddEntry = forge.ContactBookAddEntry
ContactBookClearEntries = forge.ContactBookClearEntries
ShowLetter = forge.ShowLetter
ShowInvitationCard = forge.ShowInvitationCard

class GlobalValue:
    pass

# ================= asset =================
class STATE:
    BLOCK = 0
    MOBILE = 1
    VOXMODEL = 2
    AMOUNT = 3

class INTERACT:
    MOUSE_LEFT = 0
    MOUSE_RIGHT = 1

# 全局变量
Item2Asset_ = {}
Entity2Object_ = {}

# Asset 基类
class AssetBase:
    pass

# 创建 Asset 类
def AssetCreate(baseClass = None):
    base = (AssetBase,)
    if not baseClass is None:
        if not isinstance(baseClass, tuple):
            raise TypeError('AssetCreate() argument 1 must be tuple')
        base = base + baseClass
    asset = type('Asset', base, {
        'name': None,
        'icon': '',
        'state': None,
        'blockSort': None,
        'mobSkeleton': None,
        'vmFile': None,
        'mobMesh': None,
        'mobTexture': None,
        'mobAction': None,
        'mobActionRange': None,
        'vmWrite': False,
        'onBuilds_': [],
        'onBreaks_': [],
        'onCollides_': [],
        'onInteracts_': [],
        'onStarts_': [],
        'onInterpolates_': [],
        'onUpdates_': [],
        'onKeys_': [],
        'onMouseButtons_': [],
        'onCommands_': [],
    })
    return asset

# 设置事件回调
def AssetSetBuildCBFunc_(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetBuildCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetBuildCBFunc() argument 2 must be function')
    asset.onBuilds_.append(func)
    pass
def AssetSetBreakCBFunc_(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetBreakCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetBreakCBFunc() argument 2 must be function')
    asset.onBreaks_.append(func)
    pass
def AssetSetCollideCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetCollideCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetCollideCBFunc() argument 2 must be function')
    asset.onCollides_.append(func)
    pass
def AssetSetInteractCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetInteractCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetInteractCBFunc() argument 2 must be function')
    asset.onInteracts_.append(func)
    pass
def AssetSetStartCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetStartCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetStartCBFunc() argument 2 must be function')
    asset.onStarts_.append(func)
    pass
def AssetSetInterpolateCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetInterpolateCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetInterpolateCBFunc() argument 2 must be function')
    asset.onInterpolates_.append(func)
    pass
def AssetSetUpdateCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetUpdateCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetUpdateCBFunc() argument 2 must be function')
    asset.onUpdates_.append(func)
    pass
def AssetSetKeyCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetKeyCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetKeyCBFunc() argument 2 must be function')
    asset.onKeys_.append(func)
    pass
def AssetSetMouseButtonCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetMouseButtonCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetMouseButtonCBFunc() argument 2 must be function')
    asset.onMouseButtons_.append(func)
    pass
def AssetSetCommandCBFunc(asset, func):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetSetCommandCBFunc() argument 1 must be subclass of AssetBase')
    if not util.isFunction(func):
        raise TypeError('AssetSetCommandCBFunc() argument 2 must be function')
    asset.onCommands_.append(func)
    pass

def FindAssetObject_(entity):
    global Entity2Object_
    if not entity in Entity2Object_:
        return None
    return Entity2Object_[entity]

# Build 事件响应
def OnItemBuild_(item, vmId, x, y, z, isBuild):
    global Item2Asset_
    global Entity2Object_
    if not item in Item2Asset_:
        return None
    asset = Item2Asset_[item]
    newObj = asset()
    state = newObj.state
    value = 0
    if state == STATE.BLOCK:
        value = newObj.blockSort
    elif state == STATE.MOBILE:
        value = woxiao.MobCreate(newObj.mobSkeleton, 1)
        if value is None:
            return None
        if not newObj.mobMesh is None:
            woxiao.MobLoadMesh(value, 'mesh', newObj.mobMesh)
        if not newObj.mobTexture is None:
            woxiao.MobLoadTexture(value, 'texture', newObj.mobTexture)
        if not newObj.mobAction is None:
            woxiao.MobLoadAnimation(value, 'action', newObj.mobAction)
            woxiao.MobDoAction(value, 'action', 1, True)
        if not newObj.mobActionRange is None:
            woxiao.MobSetActionRange(value, newObj.mobActionRange[0], newObj.mobActionRange[1])
    elif state == STATE.VOXMODEL:
        value = woxiao.VoxModelInsert(newObj.vmFile, wx.VOXMODEL_TYPE.MODEL, 0, isBuild or newObj.vmWrite)        
    newObj.id = value
    entity = id(newObj)
    Entity2Object_[entity] = newObj
    for func in newObj.onBuilds_:
        func(newObj)
    return [state, value, entity]

# Break 事件响应
def OnItemBreak_(entity):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onBreaks_:
        func(obj)
    if obj.state == STATE.MOBILE:
        woxiao.MobDestroy(obj.id)
    elif obj.state == STATE.VOXMODEL:
        woxiao.VoxModelRemove(obj.id)
    pass

# Collide 事件响应
def OnItemCollide_(entity, otherEntity):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    otherObj = None
    if not otherEntity is None:
        otherObj = FindAssetObject_(otherEntity)
    for func in obj.onCollides_:
        func(obj, otherObj)
    pass

# Interact 事件响应
def OnItemInteract_(entity, interact):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onInteracts_:
        func(obj, interact)
    pass

# Start 事件响应
def OnItemStart_(entity):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onStarts_:
        func(obj)
    pass

# Interpolate 事件响应
def OnInterpolate_(entity, stamp, deltaTime):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onInterpolates_:
        func(obj, stamp, deltaTime)
    pass

# Update 事件响应
def OnUpdate_(entity, stamp, deltaTime):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onUpdates_:
        func(obj, stamp, deltaTime)
    pass

# Key 事件响应
def OnKey_(entity, stamp, key, scancode, action, mods):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onKeys_:
        func(obj, stamp, key, scancode, action, mods)
    pass

# MouseButton 事件响应
def OnMouseButton_(entity, stamp, button, action, mods):
    obj = FindAssetObject_(entity)
    if obj is None:
        return
    for func in obj.onMouseButtons_:
        func(obj, stamp, button, action, mods)
    pass

# 发送指令
def SendCommand(command):
    global Entity2Object_
    for obj in Entity2Object_.values():
        for func in obj.onCommands_:
            func(obj, command)
    pass

# 注册 Asset
def AssetRegister(asset):
    if not issubclass(asset, AssetBase):
        raise TypeError('AssetRegister() argument 1 must be subclass of AssetBase')
    if not util.isString(asset.name):
        raise TypeError('attribute "name" in AssetRegister() argument 1 must be string')
    if not util.isString(asset.icon):
        raise TypeError('attribute "icon" in AssetRegister() argument 1 must be string')
    if asset.state == STATE.BLOCK:
        if not util.isInteger(asset.blockSort):
            raise TypeError('attribute "blockSort" in AssetRegister() argument 1 must be integer')
        if not woxiao.BlockSortIsExist(asset.blockSort):
            raise ValueError('block sort %d is not exist' % asset.blockSort)
    elif asset.state == STATE.MOBILE:
        if not util.isString(asset.mobSkeleton):
            raise TypeError('attribute "mobSkeleton" in AssetRegister() argument 1 must be string')
    elif asset.state == STATE.VOXMODEL:
        if not util.isString(asset.vmFile):
            raise TypeError('attribute "vmFile" in AssetRegister() argument 1 must be string')
    else:
        raise TypeError('attribute "state" in AssetRegister() argument 1 must be integer between [0, 2]')
    # 注册
    item = forge.ItemRegister(0, asset.name, asset.icon, asset.state + 1, 1 if asset.state == STATE.VOXMODEL else 0, 0)
    if item is None:
        return None
    global Item2Asset_
    Item2Asset_[item] = asset
    # 设置事件响应函数
    forge.ItemSetCallbackBuild(0, item, OnItemBuild_)
    forge.ItemSetCallbackBreak(0, item, OnItemBreak_)
    forge.ItemSetCallbackCollide(0, item, OnItemCollide_)
    forge.ItemSetCallbackInteract(0, item, OnItemInteract_)
    forge.ItemSetCallbackStart(0, item, OnItemStart_)
    forge.ItemSetCallbackInterpolate(0, item, OnInterpolate_)
    forge.ItemSetCallbackUpdate(0, item, OnUpdate_)
    forge.ItemSetCallbackKey(0, item, OnKey_)
    forge.ItemSetCallbackMouseButton(0, item, OnMouseButton_)
    return item

def AssetGetAllEntities():
    global Entity2Object_
    return Entity2Object_.values()

# 按名字查找 Entity
def AssetGetEntitiesByName(name):
    global Entity2Object_
    result = []
    for obj in Entity2Object_.values():
        if obj.name == name:
            result.append(obj)
    return result

def AssetBuildEntity(item, vm, pos):
    global Item2Asset_
    global Entity2Object_
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('AssetBuildEntity() argument 3 must be Vec3 object')
    entity = forge.ItemBuildEntity(0, item, vm, pos.x, pos.y, pos.z)
    if entity is None:
        return None
    return Entity2Object_[entity]

def AssetBreakEntity(obj):
    global Entity2Object_
    for key in Entity2Object_:
        if Entity2Object_[key] is obj:
            forge.ItemBreakEntity(0, key)
            break
    pass

# ================= SimpleMob =================
class CAMERA_CONTROL:
    FIRSTPERSON = 0
    THIRDPERSON_REAR = 1
    THIRDPERSON_FRONT = 2
    THIRDPERSON_OBLIQUE = 3

def radians_(r):
    return r * math.pi / 180

def calcYawInRange_(yaw):
    if yaw < 0:
        return yaw + radians_(360)
    elif yaw > radians_(360):
        return yaw - radians_(360)
    else:
        return yaw

class SimpleMobAction:
    STAND = 'stand'
    WALK = 'walk'
    HIT = 'hit'
    def __init__(self, begin, end, loop, priority):
        self.begin = begin
        self.end = end
        self.loop = loop
        self.priority = priority
        pass

distance_camera_to_mob = 5

class CameraControlFp:
    def init(self, mob):
        joint = woxiao.MobGetJoint(mob.id, 'camera')
        if joint < 0:
            return False
        self.mob = mob
        self.isFlying = False
        woxiao.CameraAttachMobJoint(mob.id, joint)
        woxiao.CameraSetPosition(0, 0, 0, 0, 0)
        woxiao.CameraSetFlying(True)
        return True
    def onMove(self, motion):
        if self.isFlying:
            motion = motion | wx.MOTION.NO_GRAVITY
        self.mob.setMotion(motion, True)
        pass
    def onRotate(self, deltaYaw, deltaPitch):
        (x, y, z, yaw, pitch) = woxiao.MobGetPosition(self.mob.id)
        (cx, cy, cz, cyaw, cpitch) = woxiao.CameraGetPosition()
        yaw = calcYawInRange_(yaw + deltaYaw)
        cpitch = vmath.clamp(cpitch + deltaPitch, radians_(-90), radians_(90))
        woxiao.MobSetPosition(self.mob.id, x, y, z, yaw, 0)
        woxiao.CameraSetPosition(0, 0, 0, 0, cpitch)
        pass
    def onFlyingToggle(self):
        self.isFlying = not self.isFlying
        pass

class CameraControlTpRear:
    def init(self, mob):
        global distance_camera_to_mob
        joint = woxiao.MobGetJoint(mob.id, 'camera')
        if joint < 0:
            return False
        self.mob = mob
        self.isFlying = False
        woxiao.CameraAttachMobJoint(mob.id, joint)
        woxiao.CameraSetPosition(0, 0, distance_camera_to_mob, 0, 0)
        woxiao.CameraSetFlying(True)
        return True
    def onMove(self, motion):
        if self.isFlying:
            motion = motion | wx.MOTION.NO_GRAVITY
        self.mob.setMotion(motion, True)
        pass
    def onRotate(self, deltaYaw, deltaPitch):
        global distance_camera_to_mob
        (x, y, z, yaw, pitch) = woxiao.MobGetPosition(self.mob.id)
        (cx, cy, cz, cyaw, cpitch) = woxiao.CameraGetPosition()
        yaw = calcYawInRange_(yaw + deltaYaw)
        cpitch = vmath.clamp(cpitch + deltaPitch, radians_(-45), radians_(45))
        cy = -distance_camera_to_mob * math.sin(cpitch)
        cz = distance_camera_to_mob * math.cos(cpitch)
        woxiao.MobSetPosition(self.mob.id, x, y, z, yaw, 0)
        woxiao.CameraSetPosition(0, cy, cz, 0, cpitch)
        pass
    def onFlyingToggle(self):
        self.isFlying = not self.isFlying
        pass

class CameraControlTpFront:
    def init(self, mob):
        global distance_camera_to_mob
        joint = woxiao.MobGetJoint(mob.id, 'camera')
        if joint < 0:
            return False
        self.mob = mob
        self.isFlying = False
        woxiao.CameraAttachMobJoint(mob.id, joint)
        woxiao.CameraSetPosition(0, 0, -distance_camera_to_mob, radians_(180), 0)
        woxiao.CameraSetFlying(True)
        return True
    def onMove(self, motion):
        if self.isFlying:
            motion = motion | wx.MOTION.NO_GRAVITY
        self.mob.setMotion(motion, True)
        pass
    def onRotate(self, deltaYaw, deltaPitch):
        global distance_camera_to_mob
        (x, y, z, yaw, pitch) = woxiao.MobGetPosition(self.mob.id)
        (cx, cy, cz, cyaw, cpitch) = woxiao.CameraGetPosition()
        yaw = calcYawInRange_(yaw + deltaYaw)
        cpitch = vmath.clamp(cpitch + deltaPitch, radians_(-45), radians_(45))
        cy = -distance_camera_to_mob * math.sin(cpitch)
        cz = -distance_camera_to_mob * math.cos(cpitch)
        woxiao.MobSetPosition(self.mob.id, x, y, z, yaw, 0)
        woxiao.CameraSetPosition(0, cy, cz, radians_(180), cpitch)
        pass
    def onFlyingToggle(self):
        self.isFlying = not self.isFlying
        pass

class CameraControlTpOblique:
    def __init__(self):
        self.d = distance_camera_to_mob / math.sqrt(3)
        pass
    def init(self, mob):
        global distance_camera_to_mob
        joint = woxiao.MobGetJoint(mob.id, 'camera')
        if joint < 0:
            return False
        self.mob = mob
        self.isFlying = False
        woxiao.CameraAttachMobJoint(mob.id, joint)
        woxiao.CameraSetPosition(self.d, self.d, self.d, radians_(-45), radians_(-45))
        woxiao.CameraSetFlying(True)
        return True
    def onMove(self, motion):
        if self.isFlying:
            motion = motion | wx.MOTION.NO_GRAVITY
        direction = vmath.Vec2(0, 0)
        if motion & wx.MOTION.FORWARD:
            direction = direction - vmath.Vec2(1, 1)
            motion = motion & ~wx.MOTION.FORWARD
        if motion & wx.MOTION.BACKWARD:
            direction = direction + vmath.Vec2(1, 1)
            motion = motion & ~wx.MOTION.BACKWARD
        if motion & wx.MOTION.STRAFE_LEFT:
            direction = direction + vmath.Vec2(-1, 1)
            motion = motion & ~wx.MOTION.STRAFE_LEFT
        if motion & wx.MOTION.STRAFE_RIGHT:
            direction = direction + vmath.Vec2(1, -1)
            motion = motion & ~wx.MOTION.STRAFE_RIGHT
        if direction.length() == 0:
            self.mob.setMotion(motion, True)
            return
        if direction.x > 0 and direction.y > 0:
            yaw = radians_(135)
        elif direction.x < 0 and direction.y < 0:
            yaw = radians_(-45)
        elif direction.x > 0 and direction.y < 0:
            yaw = radians_(45)
        elif direction.x < 0 and direction.y > 0:
            yaw = radians_(-135)
        elif direction.x > 0:
            yaw = radians_(90)
        elif direction.x < 0:
            yaw = radians_(-90)
        elif direction.y > 0:
            yaw = radians_(180)
        else:
            yaw = 0
        pos = wx.MobGetPosition(self.mob.id)
        woxiao.MobSetPosition(self.mob.id, pos.x, pos.y, pos.z, yaw, 0)
        (x, y, z) = woxiao.CalcPositionWithRotate(self.d, self.d, self.d, 0, 1, 0, -yaw)
        woxiao.CameraSetPosition(x, y, z, radians_(-45) - yaw, radians_(-45))
        self.mob.setMotion(motion | wx.MOTION.FORWARD, True)
        pass
    def onRotate(self, deltaYaw, deltaPitch):
        pass
    def onFlyingToggle(self):
        self.isFlying = not self.isFlying
        pass

camera_control_ = None

def OnCameraControlMove_(motion):
    global camera_control_
    if not camera_control_ is None:
        camera_control_.onMove(motion)
    pass

def OnCameraControlRotate_(deltaYaw, deltaPitch):
    global camera_control_
    if not camera_control_ is None:
        camera_control_.onRotate(deltaYaw, deltaPitch)
    pass

def OnCameraControlFlyingToggle_():
    global camera_control_
    if not camera_control_ is None:
        camera_control_.onFlyingToggle()
    pass

simpleMobs_ = []

class SimpleMob:
    def __init__(self):
        global simpleMobs_
        simpleMobs_.append(self)
        self.actionTime_ = 0
        self.actions_ = {}
        self.currentAction_ = None
        self.speed_ = 1
        self.angleSpeed_ = 1
        pass
    def __del__(self):
        global simpleMobs_
        simpleMobs_.remove(self)
        pass
    @classmethod
    def create(cls, skeleton, mesh, texture, action):
        mob = cls()
        mob.id = woxiao.MobCreate(skeleton, 1)
        if mob.id is None:
            return None
        woxiao.MobLoadMesh(mob.id, 'mesh', mesh)
        woxiao.MobLoadTexture(mob.id, 'texture', texture)
        woxiao.MobLoadAnimation(mob.id, 'action', action)
        return mob
    def setSpeed(self, speed):
        if not util.isNumber(speed) or speed < 0:
            raise TypeError('setSpeed() argument 1 must be number greater than 0')
        self.speed_ = speed
        pass
    def setAngleSpeed(self, angleSpeed):
        if not util.isNumber(angleSpeed) or angleSpeed < 0:
            raise TypeError('setAngleSpeed() argument 1 must be number greater than 0')
        self.angleSpeed_ = angleSpeed
        pass
    def setActionTime(self, time):
        if not util.isNumber(time) or time <= 0:
            raise TypeError('setActionTime() argument 1 must be number greater than 0')
        self.actionTime_ = time
        pass
    def setAction(self, name, begin, end):
        if not util.isString(name):
            raise TypeError('setAction() argument 1 must be string')
        if not util.isNumber(begin) or (begin < 0 or begin > 1):
            raise TypeError('setAction() argument 2 must be number between 0 and 1')
        if not util.isNumber(end) or (end < 0 or end > 1):
            raise TypeError('setAction() argument 3 must be number between 0 and 1')
        if begin > end:
            raise TypeError('setAction() argument 2 must not be greater than argument 3')
        if name == SimpleMobAction.STAND:
            loop = True
            priority = 0
        elif name == SimpleMobAction.WALK:
            loop = True
            priority = 0
        elif name == SimpleMobAction.HIT:
            loop = False
            priority = 2
        else:
            loop = False
            priority = 1
        self.actions_[name] = SimpleMobAction(begin, end, loop, priority)
        pass
    def doAction(self, name, force = False):
        if not util.isString(name):
            raise TypeError('doAction() argument 1 must be string')
        if not name in self.actions_:
            return False
        if not force and not self.currentAction_ is None and self.currentAction_.priority > self.actions_[name].priority:
            return False
        woxiao.MobDoAction(self.id, 'action', 1, self.actions_[name].loop)
        woxiao.MobSetActionRange(self.id, self.actions_[name].begin, self.actions_[name].end)
        self.currentAction_ = self.actions_[name]
        if not self.actions_[name].loop:
            duration = (self.actions_[name].end - self.actions_[name].begin) * self.actionTime_
            time.setDelayCommand(lambda: self.setMotion(wx.MOTION.NONE, True), duration)
        return True
    def setMotion(self, motion, force = False):
        if not util.isInteger(motion):
            raise TypeError('setMotion() argument 1 must be integer')
        self.doAction(SimpleMobAction.STAND if (motion & (wx.MOTION.FORWARD | wx.MOTION.BACKWARD | wx.MOTION.STRAFE_LEFT | wx.MOTION.STRAFE_RIGHT)) == 0 else SimpleMobAction.WALK, force)
        woxiao.MobSetMotion(self.id, motion, self.speed_, self.angleSpeed_)
        pass
    def knockback(self, attackPos):
        if not isinstance(attackPos, vmath.Vec3):
            raise TypeError('knockback() argument 1 must be Vec3 object')
        pos = wx.MobGetPosition(self.id)
        direction = pos - attackPos
        if direction.x == 0 and direction.z == 0:
            return
        # 计算受力方向和+z轴的夹角
        angle = (math.pi / 2) if direction.z == 0 else math.atan(direction.x / direction.z)
        if direction.z < 0:
            angle = math.pi - angle
        elif direction.x > 0:
            angle = math.pi * 2 - angle
        else:
            angle = - angle
        (x, y, z, yaw, pitch) = woxiao.MobGetPosition(self.id)
        angle = angle - yaw # 转化为角色坐标轴的相对yaw
        # 确保angle值范围在 0 到 2π 之间
        if angle < 0:
            angle = angle + math.pi * 2
        angle = angle * 180 / math.pi # 弧度转角度
        motion = wx.MOTION.JUMPING
        if angle < 60 or angle > 300:
            motion = motion | wx.MOTION.FORWARD
        elif angle > 120 and angle < 240:
            motion = motion | wx.MOTION.BACKWARD
        if angle > 210 and angle < 330:
            motion = motion | wx.MOTION.STRAFE_LEFT
        elif angle > 30 and angle < 150:
            motion = motion | wx.MOTION.STRAFE_RIGHT
        self.doAction(SimpleMobAction.HIT)
        woxiao.MobSetMotion(self.id, motion, 5, 0)
        time.setDelayCommand(lambda: self.setMotion(wx.MOTION.NONE, True), 0.35)
        pass
    def attachCamera(self, control):
        global camera_control_
        if control == CAMERA_CONTROL.FIRSTPERSON:
            camera_control_ = CameraControlFp()
        elif control == CAMERA_CONTROL.THIRDPERSON_REAR:
            camera_control_ = CameraControlTpRear()
        elif control == CAMERA_CONTROL.THIRDPERSON_FRONT:
            camera_control_ = CameraControlTpFront()
        elif control == CAMERA_CONTROL.THIRDPERSON_OBLIQUE:
            camera_control_ = CameraControlTpOblique()
        else:
            return False
        if camera_control_.init(self):
            forge.SetCameraControl(OnCameraControlMove_, OnCameraControlRotate_, OnCameraControlFlyingToggle_)
            return True
        else:
            return False
