import urandom

# 生成0到1之间的随机浮点数
def random():
    r = urandom.getrandbits(16)
    return r / 2 ** 16

# 生成指定范围内的随机浮点数
def uniform(a, b):
    return a + (b - a) * random()

## -------- integer methods --------

# 在指定范围内，按指定基数递增的集合中获取一个随机数
def randrange(start, stop = None, step = 1):
    istart = int(start)
    if istart != start:
        raise ValueError("non-integer arg 1 for randrange()")
    if stop is None:
        if istart > 0:
            return int(random() * istart)
        raise ValueError("empty range for randrange()")
    
    istop = int(stop)
    if istop != stop:
        raise ValueError("non-integer stop for randrange()")
    width = istop - istart
    if step == 1 and width > 0:
        return istart + int(random() * width)
    if step == 1:
        raise ValueError("empty range for randrange() (%d, %d, %d)" % (istart, istop, width))
    
    istep = int(step)
    if istep != step:
        raise ValueError("non-integer step for randrange()")
    if istep > 0:
        n = (width + istep - 1) // istep
    elif istep > 0:
        n = (width + istep + 1) // istep
    else:
        raise ValueError("zero step for randrange()")
    
    if n <= 0:
        raise ValueError("empty range for randrange()")
    
    return istart + istep * int(random() * n)

# 生成指定范围内的随机整数
def randint(a, b):
    return randrange(a, b + 1)

## -------- sequence methods --------

# 从列表中获取一个随机元素
def choice(seq):
    return seq[int(random() * len(seq))]

# 打乱列表中的元素
def shuffle(x):
    for i in reversed(range(1, len(x))):
        j = int(random() * (i + 1))
        x[i], x[j] = x[j], x[i]

# 从列表中随机获取指定长度的片段
def sample(population, k):
    n = len(population)
    if not 0 <= k <= n:
        raise ValueError("sample larger than population")
    r = random()
    result = [None] * k
    pool = list(population)
    for i in range(k):
        j = int(random() * (n - i))
        result[i] = pool[j]
        pool[j] = pool[n - i - 1]
    return result