import math
import util

def clamp(v, a, b):
    return min(max(v, a), b)

def convertNumber(v):
    try:
        n = float(v)
        if n % 1 == 0:
            n = int(n)
        return n
    except:
        return 0

class Vec2:
    def __init__(self, x, y):
        if not util.isNumber(x):
            raise TypeError('Vec2() argument 1 must be number')
        if not util.isNumber(y):
            raise TypeError('Vec2() argument 2 must be number')
        self.x = x
        self.y = y
        pass
    def __str__(self):
        return 'Vec2<' + str(self.x) + ', ' + str(self.y) + '>'
    def __repr__(self):
        return str(self)
    def __add__(self, other):
        if not isinstance(other, Vec2):
            raise TypeError('Vec2 object can only add to Vec2 object')
        obj = Vec2(self.x + other.x, self.y + other.y)
        return obj
    def __sub__(self, other):
        if not isinstance(other, Vec2):
            raise TypeError('Vec2 object can only subtract from Vec2 object')
        obj = Vec2(self.x - other.x, self.y - other.y)
        return obj
    def __mul__(self, other):
        if not util.isNumber(other):
            raise TypeError('Vec2() object can only multiply with number')
        obj = Vec2(self.x * other, self.y * other)
        return obj
    def __rmul__(self, other):
        return self * other
    def __truediv__(self, other):
        if not util.isNumber(other):
            raise TypeError('Vec2() object can only divide with number')
        obj = Vec2(self.x / other, self.y / other)
        return obj
    def __eq__(self, other):
        if not isinstance(other, Vec2):
            return False
        return self.x == other.x and self.y == other.y
    def __ne__(self, other):
        return not self == other
    def length(self):
        return math.sqrt(self.x ** 2 + self.y ** 2)
    def inner(self, a, b):
        if not isinstance(a, Vec2):
            raise TypeError('inner() argument 1 must be Vec2 object')
        if not isinstance(b, Vec2):
            raise TypeError('inner() argument 2 must be Vec2 object')
        c = (a + b) / 2
        d = (a - b) / 2
        e = self - c
        return abs(e.x) <= abs(d.x) and abs(e.y) <= abs(d.y)

class Vec3:
    def __init__(self, x, y, z):
        if not util.isNumber(x):
            raise TypeError('Vec3() argument 1 must be number')
        if not util.isNumber(y):
            raise TypeError('Vec3() argument 2 must be number')
        if not util.isNumber(z):
            raise TypeError('Vec3() argument 3 must be number')
        self.x = x
        self.y = y
        self.z = z
        pass
    def __str__(self):
        return 'Vec3<' + str(self.x) + ', ' + str(self.y) + ', ' + str(self.z) + '>'
    def __repr__(self):
        return str(self)
    def __add__(self, other):
        if not isinstance(other, Vec3):
            raise TypeError('Vec3 object can only add to Vec3 object')
        obj = Vec3(self.x + other.x, self.y + other.y, self.z + other.z)
        return obj
    def __sub__(self, other):
        if not isinstance(other, Vec3):
            raise TypeError('Vec3 object can only subtract from Vec3 object')
        obj = Vec3(self.x - other.x, self.y - other.y, self.z - other.z)
        return obj
    def __mul__(self, other):
        if not util.isNumber(other):
            raise TypeError('Vec3() object can only multiply with number')
        obj = Vec3(self.x * other, self.y * other, self.z * other)
        return obj
    def __rmul__(self, other):
        return self * other
    def __truediv__(self, other):
        if not util.isNumber(other):
            raise TypeError('Vec3() object can only divide with number')
        obj = Vec3(self.x / other, self.y / other, self.z / other)
        return obj
    def __eq__(self, other):
        if not isinstance(other, Vec3):
            return False
        return self.x == other.x and self.y == other.y and self.z == other.z
    def __ne__(self, other):
        return not self == other
    def length(self):
        return math.sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)
    def inner(self, a, b):
        if not isinstance(a, Vec3):
            raise TypeError('inner() argument 1 must be Vec3 object')
        if not isinstance(b, Vec3):
            raise TypeError('inner() argument 2 must be Vec3 object')
        c = (a + b) / 2
        d = (a - b) / 2
        e = self - c
        return abs(e.x) <= abs(d.x) and abs(e.y) <= abs(d.y) and abs(e.z) <= abs(d.z)
