import woxiao
import util
import vmath
import math

# ================= 枚举 =================
class KEY():
    SPACE = 32
    APOSTROPHE = 39
    COMMA = 44
    MINUS = 45
    PERIOD = 46
    SLASH = 47
    K_0 = 48
    K_1 = 49
    K_2 = 50
    K_3 = 51
    K_4 = 52
    K_5 = 53
    K_6 = 54
    K_7 = 55
    K_8 = 56
    K_9 = 57
    SEMICOLON = 59
    EQUAL = 61
    A = 65
    B = 66
    C = 67
    D = 68
    E = 69
    F = 70
    G = 71
    H = 72
    I = 73
    J = 74
    K = 75
    L = 76
    M = 77
    N = 78
    O = 79
    P = 80
    Q = 81
    R = 82
    S = 83
    T = 84
    U = 85
    V = 86
    W = 87
    X = 88
    Y = 89
    Z = 90
    LEFT_BRACKET = 91
    BACKSLASH = 92
    RIGHT_BRACKET = 93
    GRAVE_ACCENT = 96
    ESCAPE = 256
    ENTER = 257
    TAB = 258
    BACKSPACE = 259
    INSERT = 260
    DELETE = 261
    RIGHT = 262
    LEFT = 263
    DOWN = 264
    UP = 265
    PAGE_UP = 266
    PAGE_DOWN = 267
    HOME = 268
    END = 269
    CAPS_LOCK = 280
    SCROLL_LOCK = 281
    NUM_LOCK = 282
    PRINT_SCREEN = 283
    PAUSE = 284
    F1 = 290
    F2 = 291
    F3 = 292
    F4 = 293
    F5 = 294
    F6 = 295
    F7 = 296
    F8 = 297
    F9 = 298
    F10 = 299
    F11 = 300
    F12 = 301
    KP_0 = 320
    KP_1 = 321
    KP_2 = 322
    KP_3 = 323
    KP_4 = 324
    KP_5 = 325
    KP_6 = 326
    KP_7 = 327
    KP_8 = 328
    KP_9 = 329
    KP_DECIMAL = 330
    KP_DIVIDE = 331
    KP_MULTIPLY = 332
    KP_SUBTRACT = 333
    KP_ADD = 334
    KP_ENTER = 335
    LEFT_SHIFT = 340
    LEFT_CONTROL = 341
    LEFT_ALT = 342
    LEFT_SUPER = 343
    RIGHT_SHIFT = 344
    RIGHT_CONTROL = 345
    RIGHT_ALT = 346
    RIGHT_SUPER = 347
    MENU = 348

class MOUSE_BUTTON():
    LEFT = 0
    RIGHT = 1
    MIDDLE = 2

class ACTION():
    RELEASE = 0
    PRESS = 1
    REPEAT = 2

class MOD():
    SHIFT = 1
    CONTROL = 2
    ALT = 4
    SUPER = 8

class MOTION():
    NONE = 0
    FORWARD = 1
    BACKWARD = 2
    STRAFE_LEFT = 4
    STRAFE_RIGHT = 8
    YAW_LEFT = 16
    YAW_RIGHT = 32
    PITCH_UP = 64
    PITCH_DOWN = 128
    JUMPING = 256
    CROUCHING = 512
    NO_GRAVITY = 1024
    CLIMB = 2048

class VOXMODEL_TYPE():
    PROCEDURE_NOISE = 1
    MODEL = 3

class BLOCK():
    AIR = 0
    GRASS_BLOCK = 1
    RED_SAND = 2
    STONE_BRICKS = 3
    BRICKS = 4
    OAK_LOG = 5
    STONE = 6
    DIRT = 7
    OAK_PLANKS = 8
    COBBLESTONE = 11
    WATER = 12
    LAVA = 13
    CHEST = 14
    OAK_LEAVES = 15
    CLOUD = 16
    TALL_GRASS = 17
    DANDELION = 18
    POPPY = 19
    RED_MUSHROOM = 21
    BROWN_MUSHROOM = 22
    DEAD_BUSH = 23
    SPONGE = 24
    MELON = 25
    END_STONE = 26
    TNT = 27
    BLOCK_OF_EMERALD_OLD = 28
    FERN = 29
    BLACK_WOOL = 32
    BLUE_WOOL = 33
    BROWN_WOOL = 34
    CYAN_WOOL = 35
    GRAY_WOOL = 36
    GREEN_WOOL = 37
    LIGHT_BLUE_WOOL = 38
    LIME_WOOL = 39
    MAGENTA_WOOL = 40
    ORANGE_WOOL = 41
    PINK_WOOL = 42
    PURPLE_WOOL = 43
    RED_WOOL = 44
    LIGHT_GRAY_WOOL = 45
    WHITE_WOOL = 46
    YELLOW_WOOL = 47
    DIAMOND_ORE = 48
    REDSTONE_ORE = 49
    BOOKSHELF = 50
    MOSSY_COBBLESTONE = 51
    OBSIDIAN = 52
    WORKBENCH = 53
    FURNACE = 54
    SNOW_BLOCK = 57
    CLAY_BLOCK = 59
    JUKEBOX = 60
    CACTUS = 61
    MYCELIUM = 62
    NETHERRACK = 63
    GLOWSTONE = 64
    BEDROCK = 65
    GRAVEL = 66
    BLOCK_OF_IRON = 67
    BLOCK_OF_GOLD = 68
    BLOCK_OF_DIAMOND = 69
    GOLD_ORE = 70
    IRON_ORE = 71
    COAL_ORE = 72
    LAPIS_LAZULI_ORE = 73
    LAPIS_LAZULI_BLOCK = 74
    SANDSTONE = 75
    MOSSY_STONE_BRICKS = 76
    CRACKED_STONE_BRICKS = 77
    JACK_O_LANTERN_OFF = 79
    JACK_O_LANTERN_ON = 80
    BROWN_MUSHROOM_BLOCK = 81
    RED_MUSHROOM_BLOCK = 82
    EMERALD_ORE = 84
    SOUL_SAND = 85
    NETHER_BRICKS = 86
    DRY_FARMLAND = 87
    WET_FARMLAND = 88
    GRANITE = 91
    POLISHED_GRANITE = 92
    DIORITE = 93
    POLISHED_DIORITE = 94
    ANDESITE = 95
    POLISHED_ANDESITE = 96
    SMOOTH_RED_SANDSTONE = 97
    CHISELED_RED_SANDSTONE = 98
    RED_SANDSTONE = 99
    SAND = 100
    SMOOTH_SANDSTONE = 101
    CHISELED_SANDSTONE = 102
    BLOCK_OF_QUARTZ = 103
    QUARTZ_PILLAR = 104
    CHISELED_QUARTZ_BLOCK = 105
    DOUBLE_STONE_SLAB = 106
    END_STONE_BRICKS = 107
    CHISELED_STONE_BRICKS = 108
    PRISMARINE_BRICKS = 109
    PRISMARINE = 110
    DARK_PRISMARINE = 111
    PURPUR_PILLAR = 112
    PURPUR_BLOCK = 113
    RED_NETHER_BRICKS = 114
    BLOCK_OF_COAL = 115
    DRAGON_EGG = 116
    COARSE_DIRT = 117
    PODZOL = 118
    GRASS_PATH = 119
    SNOW_GRASS = 120
    WET_SPONGE = 121
    NETHER_QUARTZ_ORE = 122
    SEA_LANTERN = 123
    HAY_BALE = 124
    BONE_BLOCK = 125
    NETHER_WART_BLOCK = 126
    MAGMA_BLOCK = 127
    BEACON = 128
    WHEAT = 129
    COBWEB = 130
    REDSTONE_LAMP = 131
    OXEYE_DAISY = 132
    PEONY = 133
    ORANGE_TULIP = 134
    LILAC = 135
    ROSE_BUSH = 136
    WHITE_TULIP = 137
    BLUE_ORCHID = 138
    ALLIUM = 139
    AZURE_BLUET = 140
    IRON_BARS = 141
    MUSHROOM_STEM = 142
    CAULDRON = 143
    SPRUCE_PLANKS = 144
    BIRCH_PLANKS = 145
    JUNGLE_PLANKS = 146
    ACACIA_PLANKS = 147
    DARK_OAK_PLANKS = 148
    SPRUCE_LOG = 149
    BIRCH_LOG = 150
    JUNGLE_LOG = 151
    ACACIA_LOG = 152
    DARK_OAK_LOG = 153
    SPRUCE_LEAVES = 154
    BIRCH_LEAVES = 155
    JUNGLE_LEAVES = 156
    ACACIA_LEAVES = 157
    DARK_OAK_LEAVES = 158
    BLACK_CONCRETE = 159
    BLUE_CONCRETE = 160
    BROWN_CONCRETE = 161
    CYAN_CONCRETE = 162
    GRAY_CONCRETE = 163
    GREEN_CONCRETE = 164
    LIGHT_BLUE_CONCRETE = 165
    LIME_CONCRETE = 166
    MAGENTA_CONCRETE = 167
    ORANGE_CONCRETE = 168
    PINK_CONCRETE = 169
    PURPLE_CONCRETE = 170
    RED_CONCRETE = 171
    LIGHT_GRAY_CONCRETE = 172
    WHITE_CONCRETE = 173
    YELLOW_CONCRETE = 174
    BLACK_CONCRETE_POWDER = 175
    BLUE_CONCRETE_POWDER = 176
    BROWN_CONCRETE_POWDER = 177
    CYAN_CONCRETE_POWDER = 178
    GRAY_CONCRETE_POWDER = 179
    GREEN_CONCRETE_POWDER = 180
    LIGHT_BLUE_CONCRETE_POWDER = 181
    LIME_CONCRETE_POWDER = 182
    MAGENTA_CONCRETE_POWDER = 183
    ORANGE_CONCRETE_POWDER = 184
    PINK_CONCRETE_POWDER = 185
    PURPLE_CONCRETE_POWDER = 186
    RED_CONCRETE_POWDER = 187
    LIGHT_GRAY_CONCRETE_POWDER = 188
    WHITE_CONCRETE_POWDER = 189
    YELLOW_CONCRETE_POWDER = 190
    BLACK_SHULKER_BOX = 191
    BLUE_SHULKER_BOX = 192
    BROWN_SHULKER_BOX = 193
    CYAN_SHULKER_BOX = 194
    GRAY_SHULKER_BOX = 195
    GREEN_SHULKER_BOX = 196
    LIGHT_BLUE_SHULKER_BOX = 197
    LIME_SHULKER_BOX = 198
    MAGENTA_SHULKER_BOX = 199
    ORANGE_SHULKER_BOX = 200
    PINK_SHULKER_BOX = 201
    PURPLE_SHULKER_BOX = 202
    RED_SHULKER_BOX = 203
    LIGHT_GRAY_SHULKER_BOX = 204
    WHITE_SHULKER_BOX = 205
    YELLOW_SHULKER_BOX = 206
    BLACK_TERRACOTTA = 207
    BLUE_TERRACOTTA = 208
    BROWN_TERRACOTTA = 209
    CYAN_TERRACOTTA = 210
    GRAY_TERRACOTTA = 211
    GREEN_TERRACOTTA = 212
    LIGHT_BLUE_TERRACOTTA = 213
    LIME_TERRACOTTA = 214
    MAGENTA_TERRACOTTA = 215
    ORANGE_TERRACOTTA = 216
    PINK_TERRACOTTA = 217
    PURPLE_TERRACOTTA = 218
    RED_TERRACOTTA = 219
    LIGHT_GRAY_TERRACOTTA = 220
    WHITE_TERRACOTTA = 221
    YELLOW_TERRACOTTA = 222
    ENDER_CHEST = 223
    DOG_FOOD = 224
    POTATO = 225
    CARROT = 226
    TUBE_CORAL = 227
    HORN_CORAL = 228
    FIRE_CORAL = 229
    WOODEN_BOX_01 = 230
    WOODEN_BOX_02 = 231
    SPEAKER_01 = 232
    SPEAKER_02 = 233
    GLASS = 234
    BLUE_ICE = 235
    BARREL = 236
    LIME_GLAZED_TERRACOTTA = 237
    RED_GLAZED_TERRACOTTA = 238
    BLACK_GLAZED_TERRACOTTA = 239
    WHITE_GLAZED_TERRACOTTA = 240
    ORANGE_GLAZED_TERRACOTTA = 241
    BLOCK_OF_EMERALD = 242
    FRUIT_SAPLING = 243
    DECORATIVE_PLANKS_01 = 244
    DECORATIVE_PLANKS_02 = 245
    DECORATIVE_PLANKS_03 = 246

# ================= 内部方法 =================
def isBlockSort_(v):
    try:
        return woxiao.BlockSortIsExist(v)
    except:
        return False

# ================= 函数封装 =================
BlockSortGetByName = woxiao.BlockSortGetByName
GetHighest = woxiao.GetHighest
VoxModelSetMotion = woxiao.VoxModelSetMotion
FogSetDistantRatio = woxiao.FogSetDistantRatio
LoadSky = woxiao.LoadSky
GetFov = woxiao.GetFov
SetFov = woxiao.SetFov
CameraAttachVoxModel = woxiao.CameraAttachVoxModel
CameraSetMotion = woxiao.CameraSetMotion
CameraSetFlying = woxiao.CameraSetFlying
MobSetMotion = woxiao.MobSetMotion
MobAttachVoxModel = woxiao.MobAttachVoxModel
MobGetJoint = woxiao.MobGetJoint
MobAttachVM2Joint = woxiao.MobAttachVM2Joint
AudioCreate = woxiao.AudioCreate
AudioDestroy = woxiao.AudioDestroy
AudioPlay = woxiao.AudioPlay
AudioStop = woxiao.AudioStop
AudioStopAll = woxiao.AudioStopAll
AudioSetMaxDistance = woxiao.AudioSetMaxDistance
AudioSetGain = woxiao.AudioSetGain
AudioSetPitch = woxiao.AudioSetPitch
AudioListenerSetOri = woxiao.AudioListenerSetOri

# ================= block =================
def SetBlock(vm, pos, sort):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('SetBlock() argument 2 must be Vec3 object')
    woxiao.SetBlock(vm, pos.x, pos.y, pos.z, sort)
    pass

def BlockSortGet(vm, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('BlockSortGet() argument 2 must be Vec3 object')
    return woxiao.BlockSortGet(vm, pos.x, pos.y, pos.z)

def GetHitBlockFromCamera(distance):
    (sort, vm, x, y, z) = woxiao.GetHitBlockFromCamera(distance)
    return (sort, vm, vmath.Vec3(x, y, z))

def BuildTreeDefault(vm, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('BuildTreeDefault() argument 2 must be Vec3 object')
    woxiao.BuildTreeDefault(vm, pos.x, pos.y, pos.z)
    pass

def BuildByImage(vm, pos, file):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('BuildByImage() argument 2 must be Vec3 object')
    woxiao.BuildByImage(vm, pos.x, pos.y, pos.z, file)
    pass

def BuildBySqlite(vm, pos, file):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('BuildBySqlite() argument 2 must be Vec3 object')
    woxiao.BuildBySqlite(vm, pos.x, pos.y, pos.z, file)
    pass

def BuildBox(vm, a, b, sort):
    if not util.isInteger(vm):
        raise TypeError('BuildBox() argument 1 must be voxel model')
    if not isinstance(a, vmath.Vec3):
        raise TypeError('BuildBox() argument 2 must be Vec3 object')
    if not isinstance(b, vmath.Vec3):
        raise TypeError('BuildBox() argument 3 must be Vec3 object')
    if not isBlockSort_(sort):
        raise TypeError('BuildBox() argument 4 must be block sort')
    minX = min(a.x, b.x)
    maxX = max(a.x, b.x)
    minY = min(a.y, b.y)
    maxY = max(a.y, b.y)
    minZ = min(a.z, b.z)
    maxZ = max(a.z, b.z)
    for x in range(minX, maxX + 1):
        for y in range(minY, maxY + 1):
            for z in range(minZ, maxZ + 1):
                woxiao.SetBlock(vm, x, y, z, sort)
    pass

def BuildCylinder(vm, pos, h, r, sort, fill):
    if not util.isInteger(vm):
        raise TypeError('BuildCylinder() argument 1 must be voxel model')
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('BuildCylinder() argument 2 must be position')
    if not util.isNumber(h) or h < 0:
        raise TypeError('BuildCylinder() argument 3 must be number greater than 0')
    if not util.isNumber(r) or r < 0:
        raise TypeError('BuildCylinder() argument 4 must be number greater than 0')
    if not isBlockSort_(sort):
        raise TypeError('BuildCylinder() argument 5 must be block sort')
    for x in range(pos.x - r, pos.x + r + 1):
        for z in range(pos.z - r, pos.z + r + 1):
            v = vmath.Vec3(x, pos.y, z) - pos
            l = int(v.length())
            if l > r or (not fill and l != r):
                continue
            for y in range(pos.y, pos.y + h):                
                woxiao.SetBlock(vm, x, y, z, sort)
    pass

# ================= world =================
def SetDayTime(time):
    (old, length) = woxiao.GetDayTime()
    woxiao.SetDayTime(time, length)
    pass

def SetDayLength(length):
    (time, old) = woxiao.GetDayTime()
    print(time)
    woxiao.SetDayTime(time * 24, length)
    pass

def GetDayTime():
    return woxiao.GetDayTime()[0] * 24

def GetDayLength():
    return woxiao.GetDayTime()[1]

def LightingBlock(vm, pos, light):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('LightingBlock() argument 2 must be Vec3 object')
    woxiao.LightingBlock(vm, pos.x, pos.y, pos.z, light)
    pass

def ParticleFountainEmit(pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('ParticleFountainEmit() argument 1 must be Vec3 object')
    woxiao.ParticleFountainEmit(pos.x, pos.y, pos.z, 0)
    pass

# ================= audio =================
def AudioSetPos(audio, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('AudioSetPos() argument 2 must be Vec3 object')
    woxiao.AudioSetPos(audio, pos.x, pos.y, pos.z)
    pass

def AudioListenerSetPos(pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('AudioListenerSetPos() argument 1 must be Vec3 object')
    woxiao.AudioListenerSetPos(pos.x, pos.y, pos.z)
    pass

# ================= camera =================
def CameraSetPosition(pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('CameraSetPosition() argument 1 must be Vec3 object')
    (x, y, z, yaw, pitch) = woxiao.CameraGetPosition()
    woxiao.CameraSetPosition(pos.x, pos.y, pos.z, yaw, pitch)
    pass

def CameraSetYaw(v):
    (x, y, z, yaw, pitch) = woxiao.CameraGetPosition()
    woxiao.CameraSetPosition(x, y, z, v, pitch)
    pass

def CameraSetPitch(v):
    (x, y, z, yaw, pitch) = woxiao.CameraGetPosition()
    woxiao.CameraSetPosition(x, y, z, yaw, v)
    pass

def CameraGetPosition():
    (x, y, z, yaw, pitch) = woxiao.CameraGetPosition()
    return vmath.Vec3(x, y, z)

def CameraGetYaw():
    return woxiao.CameraGetPosition()[3]

def CameraGetPitch():
    return woxiao.CameraGetPosition()[4]

# ================= voxmodel =================
def VoxModelSetPosition(vm, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('VoxModelSetPosition() argument 2 must be Vec3 object')
    (x, y, z, yaw, pitch) = woxiao.VoxModelGetPosition(vm)
    woxiao.VoxModelSetPosition(vm, pos.x, pos.y, pos.z, yaw, pitch)
    pass

def VoxModelSetYaw(vm, v):
    (x, y, z, yaw, pitch) = woxiao.VoxModelGetPosition(vm)
    woxiao.VoxModelSetPosition(vm, x, y, z, v, pitch)
    pass

def VoxModelSetPitch(vm, v):
    (x, y, z, yaw, pitch) = woxiao.VoxModelGetPosition(vm)
    woxiao.VoxModelSetPosition(vm, x, y, z, yaw, v)
    pass

def VoxModelGetPosition(vm):
    (x, y, z, yaw, pitch) = woxiao.VoxModelGetPosition(vm)
    return vmath.Vec3(x, y, z)

def VoxModelGetYaw(vm):
    return woxiao.VoxModelGetPosition(vm)[3]

def VoxModelGetPitch(vm):
    return woxiao.VoxModelGetPosition(vm)[4]

# ================= mob =================
def MobSetPosition(mob, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('MobSetPosition() argument 2 must be Vec3 object')
    (x, y, z, yaw, pitch) = woxiao.MobGetPosition(mob)
    woxiao.MobSetPosition(mob, pos.x, pos.y, pos.z, yaw, pitch)
    pass

def MobSetYaw(mob, v):
    (x, y, z, yaw, pitch) = woxiao.MobGetPosition(mob)
    woxiao.MobSetPosition(mob, x, y, z, v, pitch)
    pass

def MobSetPitch(mob, v):
    (x, y, z, yaw, pitch) = woxiao.MobGetPosition(mob)
    woxiao.MobSetPosition(mob, x, y, z, yaw, v)
    pass

def MobGetPosition(mob):
    (x, y, z, yaw, pitch) = woxiao.MobGetPosition(mob)
    return vmath.Vec3(x, y, z)

def MobGetYaw(mob):
    return woxiao.MobGetPosition(mob)[3]

def MobGetPitch(mob):
    return woxiao.MobGetPosition(mob)[4]

def MobDoAction(mob, begin, end, speed, loop):
    woxiao.MobSetActionRange(mob, begin, end)
    woxiao.MobDoAction(mob, 'action', speed, loop)
    pass

def MobCollidePoint(mob, pos):
    if not isinstance(pos, vmath.Vec3):
        raise TypeError('MobCollidePoint() argument 2 must be Vec3 object')
    return woxiao.MobCollidePoint(mob, pos.x, pos.y, pos.z)

def MobSetActive(mob, isActive):
    woxiao.MobSetVisible(mob, isActive)
    woxiao.MobSetEnable(mob, isActive)
    pass